import { isLineSegment, lineSegment, point } from "../math";
import { isBounds } from "./element/typeChecks";
export const debugDrawLine = (segment, opts) => {
    const segments = (isLineSegment(segment) ? [segment] : segment);
    segments.forEach((data) => addToCurrentFrame({
        color: opts?.color ?? "red",
        data,
        permanent: !!opts?.permanent,
    }));
};
export const debugDrawPoint = (p, opts) => {
    const xOffset = opts?.fuzzy ? Math.random() * 3 : 0;
    const yOffset = opts?.fuzzy ? Math.random() * 3 : 0;
    debugDrawLine(lineSegment(point(p[0] + xOffset - 10, p[1] + yOffset - 10), point(p[0] + xOffset + 10, p[1] + yOffset + 10)), {
        color: opts?.color ?? "cyan",
        permanent: opts?.permanent,
    });
    debugDrawLine(lineSegment(point(p[0] + xOffset - 10, p[1] + yOffset + 10), point(p[0] + xOffset + 10, p[1] + yOffset - 10)), {
        color: opts?.color ?? "cyan",
        permanent: opts?.permanent,
    });
};
export const debugDrawBoundingBox = (box, opts) => {
    (Array.isArray(box) ? box : [box]).forEach((bbox) => debugDrawLine([
        lineSegment(point(bbox.minX, bbox.minY), point(bbox.maxX, bbox.minY)),
        lineSegment(point(bbox.maxX, bbox.minY), point(bbox.maxX, bbox.maxY)),
        lineSegment(point(bbox.maxX, bbox.maxY), point(bbox.minX, bbox.maxY)),
        lineSegment(point(bbox.minX, bbox.maxY), point(bbox.minX, bbox.minY)),
    ], {
        color: opts?.color ?? "cyan",
        permanent: opts?.permanent,
    }));
};
export const debugDrawBounds = (box, opts) => {
    (isBounds(box) ? [box] : box).forEach((bbox) => debugDrawLine([
        lineSegment(point(bbox[0], bbox[1]), point(bbox[2], bbox[1])),
        lineSegment(point(bbox[2], bbox[1]), point(bbox[2], bbox[3])),
        lineSegment(point(bbox[2], bbox[3]), point(bbox[0], bbox[3])),
        lineSegment(point(bbox[0], bbox[3]), point(bbox[0], bbox[1])),
    ], {
        color: opts?.color ?? "green",
        permanent: !!opts?.permanent,
    }));
};
export const debugCloseFrame = () => {
    window.visualDebug?.data.push([]);
};
export const debugClear = () => {
    if (window.visualDebug?.data) {
        window.visualDebug.data = [];
    }
};
const addToCurrentFrame = (element) => {
    if (window.visualDebug?.data && window.visualDebug.data.length === 0) {
        window.visualDebug.data[0] = [];
    }
    window.visualDebug?.data &&
        window.visualDebug.data[window.visualDebug.data.length - 1].push(element);
};
